@@ -1,8 +1,9 @@
 #!/bin/sh
-# $Id: run-tests.sh 333 2003-07-08 00:02:08Z miles $
 
 CMDLINE="sbcl --noinform --load xmls --eval"
 FORM="(xmls::test)"
+ALLEGROCMD="/usr/local/acl/acl62/lisp"
+SEPARATOR=""
 
 usage () {
     cat <<USAGE
@@ -12,6 +13,7 @@ options:
     --cmucl run tests with cmucl
     --all run all tests in tests directory
     --verbose output parsed xml
+    --acl62 run tests with Allegro Common Lisp, v6.2
 USAGE
     exit 1
 }
@@ -22,6 +24,11 @@ while [ $# -gt 0 ]; do
             CMDLINE="lisp -load xmls -eval"
             shift
             ;;
+        --acl62)
+            CMDLINE="${ALLEGROCMD} -L xmls -e"
+            SEPARATOR="--"
+            shift
+            ;;
         --all)
             TESTS="tests/*/*"
             shift
@@ -41,4 +48,4 @@ if test -z "$TESTS"; then
     usage
 fi
 
-$CMDLINE "$FORM" $TESTS
+$CMDLINE "$FORM" $SEPARATOR $TESTS
