@@ -1,4 +1,7 @@
-;;; $Id: xmls.lisp 447 2003-08-29 04:40:29Z miles $
+;;; xmls
+;;; a simple xml parser for common lisp
+;;; author: Miles Egan <miles@caddr.com>
+;;; see COPYING file for license information
 
 (defpackage xmls
   (:use :cl :cl-user)
@@ -12,20 +15,41 @@
 (defvar *strip-comments* t)
 (defvar *compress-whitespace* t)
 (defvar *test-verbose* nil)
-(defvar *whitespace* '(#\Newline #\Space #\Tab #\Return #\Linefeed))
 (defvar *entities*
-  '(("lt;" #\<)
+  #(("lt;" #\<)
     ("gt;" #\>)
     ("amp;" #\&)
     ("apos;" #\')
     ("quot;" #\")))
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defvar *whitespace* (remove-duplicates 
+                        '(#\Newline #\Space #\Tab #\Return #\Linefeed))))
+(defvar *char-escapes*
+  (let ((table (make-array 256 :element-type 'string :initial-element "")))
+    (declare (type vector *entities*))
+    (loop
+     for code from 0 to 255
+     for char = (code-char code)
+     for entity = (first (find char *entities* :test #'char= :key #'second))
+     do (setf (svref table code)
+              (cond
+                (entity
+                 (concatenate 'string "&" entity))
+                ((and (or (< code 32) (> code 126))
+                      (not (= code 10))
+                      (not (= code 9)))
+                 (format nil "&#x~x;" code))
+                (t
+                 (format nil "~x" char))))
+     finally (return table))
+    table))
 
 ;;;-----------------------------------------------------------------------------
 ;;; CONDITIONS
 ;;;-----------------------------------------------------------------------------
 (define-condition xml-parse-error (error) 
   ((line :initarg :line
-         :reader :error-line)))
+         :reader error-line)))
 
 ;;;-----------------------------------------------------------------------------
 ;;; NODE INTERFACE
@@ -76,38 +100,13 @@
             str))
       str))
   
-(defun entity-of (char)
-  "Returns the xml entity corresponding to CHAR, without the leading
-ampersand. Returns NIL if not found."
-  (declare (type list *entities*))
-  (first (find char *entities* 
-	       :test #'char= 
-	       :key (lambda (e) (second e)))))
-
-(defun entitify (object)
-  "Converts OBJECT to its string representation, if necessary, and then replaces 
-the characters of OBJECT with their corresponding entities. Assumes that the 
-characters of RESERVED have been registered in the entity table."
-  (let ((str (typecase object
-               (string (coerce object 'simple-base-string))
-               (t (format nil "~A" object)))))
-    (declare (type simple-base-string str)) ;; mollify sbcl compiler
-    (loop with stream = (make-string-output-stream)
-          for c across str
-          for ent = (entity-of c)
-          for code = (char-code c)
-          do (cond
-               (ent
-                (progn
-                  (write-char #\& stream)
-                  (write-string ent stream)))
-               ((and (or (< code 32) (> code 126))
-                     (not (= code 10))
-                     (not (= code 9)))
-                (format stream "&#x~x;" code))
-               (t
-                (write-char c stream)))
-          finally (return (get-output-stream-string stream)))))
+(defun write-escaped (string stream)
+  "Writes string to stream with all character entities escaped."
+  (coerce string 'simple-base-string)
+  (when (eq stream t) (setf stream *standard-output*))
+  (loop for char across string
+        for esc = (svref *char-escapes* (char-code char))
+        do (write-sequence esc stream)))
 
 (defun make-extendable-string (&optional (size 10))
   "Creates an adjustable string with a fill pointer."
@@ -120,21 +119,31 @@ characters of RESERVED have been registered in the entity table."
   "Shorthand function for adding characters to an extendable string."
   (vector-push-extend c string))
 
+(defun translate-raw-value (raw-value)
+  "Helper function for xml generation."
+  (etypecase raw-value
+    (string raw-value)
+    (symbol (symbol-name raw-value))
+    (integer (format nil "~D" raw-value))
+    (float (format nil "~G" raw-value))))
+
 (defun generate-xml (e s indent)
   "Renders a lisp node tree to an xml string stream."
   (if (> indent 0) (incf indent))
-  (typecase e
+  (etypecase e
     (list
      (progn
        (dotimes (i (* 2 (- indent 2)))
          (write-char #\Space s))
-       (format s "<~A~@[ xmlns=\"~A\"~]~:{ ~A=\"~A\"~}"
-               (node-name e)
-               (node-ns e)
-               (mapcar (lambda (attr) 
-                         (list (first attr)
-                               (entitify (second attr))))
-                       (node-attrs e))))
+       (format s "<~A~@[ xmlns=\"~A\"~]" (node-name e) (node-ns e))
+       (loop for a in (node-attrs e)
+             do (progn
+                  (write-char #\Space s)
+                  (write-string (first a) s)
+                  (write-char #\= s)
+                  (write-char #\" s)
+                  (write-escaped (second a) s)
+                  (write-char #\" s))))
      (if (null (node-children e))
          (progn
            (write-string "/>" s)
@@ -149,13 +158,17 @@ characters of RESERVED have been registered in the entity table."
                    (write-char #\Space s))))
 	   (format s "</~A>" (node-name e))
            (if (> indent 0) (write-char #\Newline s)))))
-    (base-string 
+    (number
+     (generate-xml (translate-raw-value e) s indent))
+    (symbol
+     (generate-xml (translate-raw-value e) s indent))
+    (string 
      (progn
        (if (> indent 0)
            (progn
              (dotimes (i (* 2 (- indent 2)))
                (write-char #\Space s))))
-       (write-string (entitify e) s)
+       (write-escaped e s)
        (if (> indent 0) (write-char #\Newline s))))))
 
 ;;;-----------------------------------------------------------------------------
@@ -173,13 +186,14 @@ characters of RESERVED have been registered in the entity table."
 converted using CODE-CHAR, which only works in implementations that
 internally encode strings in US-ASCII, ISO-8859-1 or UCS."
   (declare (type simple-base-string ent))
+  (declare (type vector *entities*))
   (or (and (>= (length ent) 2)
            (char= (char ent 0) #\#)
            (code-char
             (if (char= (char ent 1) #\x)
                 (parse-integer ent :start 2 :end (- (length ent) 1) :radix 16)
                 (parse-integer ent :start 1 :end (- (length ent) 1)))))
-      (second (assoc ent *entities* :test #'string=))
+      (second (find ent *entities* :test #'string= :key #'first))
       (error "Unable to resolve entity ~S" ent)))
 
 (declaim (inline peek-stream))
@@ -314,7 +328,7 @@ character translation."
 
 (defmatch ws-char ()
   (case c
-    ((#\Newline #\Space #\Tab #\Return #\Linefeed) t)
+    (#.*whitespace* t)
     (t nil)))
 
 (defmatch namechar ()
@@ -367,14 +381,20 @@ character translation."
      (setf (values name suffix) (qname s))
      (or
       (and
-       (match #\=)
+       (progn
+         (match* ws-char)
+         (match #\=))
        (or
         (and
-         (match #\")
+         (progn
+           (match* ws-char)
+           (match #\"))
          (setf val (match* attr-text-dq))
          (match #\"))
         (and
-         (match #\')
+         (progn
+           (match* ws-char)
+           (match #\'))
          (setf val (match* attr-text-sq))
          (match #\'))))
       t)
@@ -459,7 +479,9 @@ character translation."
                                   (push-string #\] data)
                                   (push-string (eat) data)))))
                     until (eq state 3)
-                    finally (return (make-element :type 'cdata :val data))))))))
+                    finally (return (make-element 
+                                     :type 'cdata 
+                                     :val (coerce data 'simple-base-string)))))))))
 
 (declaim (ftype function element))     ; forward decl for content rule
 (defrule content ()
@@ -588,12 +610,14 @@ character translation."
 ;;(trace end-tag comment comment-or-doctype content name xmldecl misc)
 ;;(trace processing-instruction processing-instruction-or-xmldecl element start-tag ws element-val)
 
+#+(or sbcl cmu allegro)
 (defun test ()
   ;;(sb-profile:profile "XMLS")
   #+cmu(extensions:gc-off) ;; too noisy
   (dolist (test (cdr 
                  #+sbcl sb-ext:*posix-argv* 
-                 #+cmu (subseq extensions:*command-line-strings* 4)))
+                 #+cmu (subseq extensions:*command-line-strings* 4)
+                 #+allegro (sys:command-line-arguments)))
     (if *test-verbose*
         (format t "~A~%" (toxml (parse (open test) :compress-whitespace t) :indent t))
         (progn
@@ -603,4 +627,5 @@ character translation."
               (format t "failed!~%")))))
   ;;(sb-profile:report)
   #+sbcl(sb-ext:quit)
-  #+cmu(extensions:quit))         
+  #+cmu(extensions:quit)
+  #+allegro(excl:exit))
